﻿using System;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Net.Http;
using System.IO;
using Newtonsoft.Json;

namespace TeemoTool
{
    public partial class FormMain : Form
    {
        private Thread TRepeat;
        private bool active;
        private string palletPrint;
        private string host = "http://10.10.1.3:8000/";
        private string stationId;
        private string WO;
        private string palletId;
        private string palletInput;
        private string cartonId;
        private string cartonInput;
        private const string NULL = "NULL";
        private const string OK = "OK";
        private int giftQty = -1;
        private int carQty;
        private int carQtyMax = -1;
        private long timeGet;
        private static readonly HttpClient client = new HttpClient();

        public FormMain()
        {
            InitializeComponent();
            Control.CheckForIllegalCrossThreadCalls = false;
            LoadConfig();
            TRepeat = new Thread(Repeat);
            TRepeat.IsBackground = true;
            TRepeat.Start();
        }

        private long time()
        {
            return DateTime.Now.Ticks / TimeSpan.TicksPerMillisecond;
        }

        private bool CheckTB()
        {
            int count = 0;
            for (int i = 0; i < IDStationTB.TextLength; i++)
            {
                if (IDStationTB.Text[i] == '_')
                {
                    count++;
                }
            }
            return count == 2;
        }

        private void LogUpdate(string log, bool ok)
        {
            LogText.Text = ok ? log : "Lỗi kết nối hệ thống. Đang tiến hành thử lại...";
            LogText.ForeColor = ok ? System.Drawing.Color.Blue : System.Drawing.Color.Firebrick;
        }

        private int GetCartonMax(string info)
        {
            string split = info.Substring(info.IndexOf("\"CartonInPalletQty\"") + 20);
            split = split.Substring(0, split.IndexOf(','));
            return int.Parse(split);
        }

        private void SendPrint()
        {
            LogUpdate("Tiến hành gửi lệnh in tem...", true);
            SendKeys.SendWait("^a");
            SendKeys.SendWait(palletPrint);
            SendKeys.SendWait("~");
        }

        private void Repeat()
        {
            while (true)
            {
                if (active)
                {
                    CheckCarton();
                }
                Thread.Sleep(1000);
            }
        }

        private void LoadConfig()
        {
            WO = NULL;
            palletId = NULL;
            palletInput = NULL;
            cartonId = NULL;
            string[] config = File.ReadAllText("Config").Split('|');
            stationId = config[0];
            palletPrint = config[1];
            IDStationTB.Text = stationId;
            PrintCmd.Checked = palletPrint.Equals(NULL) ? false : true;
        }

        private void GetWO()
        {
            try
            {
                string[] info = stationId.Split('_');
                string payload = "{\"line_name\":\"" + info[0] + "\"}";
                var t = Task.Run(() => Post(host + "shopfloor/workoders/getLineInformation/", payload));
                t.Wait();
                if (t.Result[0].Equals(OK))
                {
                    Line var = JsonConvert.DeserializeObject<Line>(t.Result[1]);
                    WO = var.WO_activate;
                    LogUpdate("Đã nhận thông tin WO " + WO, true);
                    var v = Task.Run(() => GetPrdInfo(var.SKUcode));
                    v.Wait();
                    if (v.Result[0].Equals(OK))
                    {
                        carQtyMax = GetCartonMax(v.Result[1]);
                        LogUpdate("Đã nhận thông tin Model", true);
                    }
                }
            }
            catch
            {
                if (active)
                {
                    LogUpdate(NULL, false);
                    GetWO();
                }
            }
        }

        private void GetPalletID()
        {
            if (WO.Equals(NULL) || carQtyMax == -1)
            {
                GetWO();
                if (WO.Equals(NULL) || carQtyMax == -1)
                {
                    return;
                }
            }
            try
            {
                string payload = "{\"wo\":\"" + WO + "\"}";
                var t = Task.Run(() => Post(host + "shopfloor/pallet/getPalletOpenByWO/", payload));
                t.Wait();
                if (t.Result[0].Equals(OK))
                {
                    GetPallet var = JsonConvert.DeserializeObject<GetPallet>(t.Result[1]);
                    palletId = var.data.palletSn;
                    carQty = var.data.cartonSn.Length;
                    if (palletInput.Equals(NULL))
                    {
                        palletInput = palletId;
                    }
                    LogUpdate("Đã nhận thông tin Pallet " + palletId, true);
                }
                else
                {
                    WO = NULL;
                    carQtyMax = -1;
                }
            }
            catch
            {
                if (active)
                {
                    LogUpdate(NULL, false);
                    GetPalletID();
                }
            }
        }

        private void CheckCarton()
        {
            if (palletId.Equals(NULL))
            {
                GetPalletID();
                if (palletId.Equals(NULL))
                {
                    return;
                }
            }
            try
            {
                string payload = "{\"wo\":\"" + WO + "\"}";
                var t = Task.Run(() => Post(host + "shopfloor/carton/getCartonOpenByWO/", payload));
                t.Wait();
                if (t.Result[0].Equals(OK))
                {
                    GetCarton var = JsonConvert.DeserializeObject<GetCarton>(t.Result[1]);
                    string carton = var.data.cartonSn;
                    int qty = var.data.giftSn.Length;
                    if (cartonId.Equals(NULL))
                    {
                        cartonId = carton;
                        return;
                    }
                    if (!carton.Equals(cartonId))
                    {
                        cartonInput = cartonId;
                        cartonId = carton;
                        InputCarton();
                        return;
                    }
                    else
                    {
                        if (qty != giftQty)
                        {
                            giftQty = qty;
                            timeGet = time();
                            LogUpdate("Đã cập nhật số lượng Gift Box", true);
                        }
                        else if (time() - timeGet > 60000)
                        {
                            palletId = NULL;
                            WO = NULL;
                            carQtyMax = -1;
                            timeGet = time();
                            LogUpdate("Tiến hành kiểm tra lại thông tin...", true);
                        }
                    }
                }
            }
            catch
            {
                if (active)
                {
                    LogUpdate(NULL, false);
                    CheckCarton();
                }
            }
        }

        private void InputCarton()
        {
            try
            {
                string[] info = stationId.Split('_');
                string payload = "{\"station_name\":\"" + info[1] + "\",\"line_id\":\"" + info[0] + "\",\"station_index\":" + info[2] + ",\"wo\":\"" + WO + "\",\"cartonSn\":\"" + cartonInput + "\",\"palletSn\":\"" + palletId + "\"}";
                var t = Task.Run(() => Post(host + "shopfloor/pallet/inputCartonToPallet/", payload));
                t.Wait();
                if (t.Result[0].Equals(OK))
                {
                    giftQty = -1;
                    palletInput = palletId;
                    LogUpdate("Đã thêm Carton " + cartonInput, true);
                    carQty++;
                    if (carQty >= carQtyMax)
                    {
                        palletId = NULL;
                        if (!palletPrint.Equals(NULL))
                        {
                            palletPrint = palletInput;
                            SendPrint();
                        }
                        LogUpdate("Đang chuyển sang Pallet mới...", true);
                    }
                }
            }
            catch
            {
                if (active)
                {
                    LogUpdate(NULL, false);
                    InputCarton();
                }
            }
        }

        private async Task<string[]> GetPrdInfo(string sku)
        {
            var message = await client.GetAsync(host + "shopfloor/products/?sku_code=" + sku);
            var result = await message.Content.ReadAsStringAsync();
            return new string[] { message.StatusCode.ToString(), result };
        }

        private async Task<string[]> Post(string url, string payload)
        {
            HttpContent content = new StringContent(payload, System.Text.Encoding.UTF8, "application/json");
            var message = await client.PostAsync(url, content);
            var result = await message.Content.ReadAsStringAsync();
            return new string[] { message.StatusCode.ToString(), result };
        }

        private void FormMain_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Space)
            {
                active = !active;
                CoderText.ForeColor = active ? System.Drawing.Color.Green : System.Drawing.Color.Gray;
                LogUpdate("Công cụ " + (active ? "đang hoạt động" : "đã tạm dừng"), true);
            }
        }

        private void FormMain_Resize(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                NotifyIcon.Visible = true;
                this.Hide();
                if (active)
                {
                    NotifyIcon.ShowBalloonTip(1000);
                }
            }
        }

        private void NotifyIcon_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                NotifyIcon.Visible = false;
                this.Show();
                this.WindowState = FormWindowState.Normal;
            }
        }

        private void IDStationTB_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                IDStationTB.SelectAll();
                if (CheckTB())
                {
                    stationId = IDStationTB.Text;
                    File.WriteAllText("Config", stationId + "|" + palletPrint);
                    MessageBox.Show("Lưu thiết lập thành công!");
                }
                else
                {
                    IDStationTB.Text = "L1_Station_1";
                    MessageBox.Show("Vui lòng nhập đúng định dạng!\r\nVí dụ: L1_Station_1");
                }
            }
        }

        private void PrintCmd_Click(object sender, EventArgs e)
        {
            PrintCmd.Checked = !PrintCmd.Checked;
            if (!PrintCmd.Checked)
            {
                palletPrint = NULL;
            }
            else if (palletPrint.Equals(NULL))
            {
                palletPrint = "PALLET";
            }
            File.WriteAllText("Config", stationId + "|" + palletPrint);
            MessageBox.Show("Đã " + (PrintCmd.Checked ? "kích hoạt " : "vô hiệu ") + "gửi lệnh in tem!");
        }
    }

    public class GetCarton
    {
        public string result { get; set; }
        public string message { get; set; }
        public Carton data { get; set; }
    }

    public class Carton
    {
        public string cartonSn { get; set; }
        public string[] giftSn { get; set; }
        public string SSCC { get; set; }
    }

    public class Line
    {
        public string line_name { get; set; }
        public string status { get; set; }
        public string leader { get; set; }
        public string[] op_list { get; set; }
        public string WO_activate { get; set; }
        public int WO_producted { get; set; }
        public string WO_qty { get; set; }
        public string product_name { get; set; }
        public string SKUcode { get; set; }
        public string PO_id { get; set; }
        public int PO_producted { get; set; }
        public string PO_qty { get; set; }
    }

    public class GetPallet
    {
        public string result { get; set; }
        public string message { get; set; }
        public Pallet data { get; set; }
    }

    public class Pallet
    {
        public string palletSn { get; set; }
        public string[] cartonSn { get; set; }
        public string SSCC { get; set; }
    }
}
